@@ -75,7 +75,9 @@ pub enum PixelKind {
     D16,
     D24S8,
     RGBA8,
+    SRGBA8,
     RGB8,
+    SRGB8,
     BGRA8,
     BGR8,
     RG8,
@@ -90,8 +92,11 @@ pub enum PixelKind {
     DXT3RGBA,
     DXT5RGBA,
     RGBA32F,
+    RGBA16F,
     R8RGTC,
     RG8RGTC,
+    R11G11B10F,
+    RGB10A2,
 }
 
 impl From<TexturePixelKind> for PixelKind {
@@ -125,26 +130,30 @@ pub enum PixelElementKind {
 }
 
 impl PixelKind {
-    pub fn unpack_alignment(self) -> i32 {
+    pub fn unpack_alignment(self) -> Option<i32> {
         match self {
-            Self::RGBA16 | Self::RGB16 | Self::RGBA32F => 8,
+            Self::RGBA16 | Self::RGBA16F | Self::RGB16 | Self::RGBA32F => Some(8),
             Self::RGBA8
+            | Self::SRGBA8
             | Self::RGB8
+            | Self::SRGB8
             | Self::BGRA8
             | Self::BGR8
             | Self::RG16
             | Self::R16
             | Self::D24S8
             | Self::D32F
-            | Self::F32 => 4,
-            Self::RG8 | Self::D16 | Self::F16 => 2,
-            Self::R8 | Self::R8UI => 1,
+            | Self::F32
+            | Self::R11G11B10F
+            | Self::RGB10A2 => Some(4),
+            Self::RG8 | Self::D16 | Self::F16 => Some(2),
+            Self::R8 | Self::R8UI => Some(1),
             Self::DXT1RGB
             | Self::DXT1RGBA
             | Self::DXT3RGBA
             | Self::DXT5RGBA
             | Self::R8RGTC
-            | Self::RG8RGTC => unreachable!(),
+            | Self::RG8RGTC => None,
         }
     }
 
@@ -159,9 +168,12 @@ impl PixelKind {
             // Explicit match for rest of formats instead of _ will help to not forget
             // to add new entry here.
             Self::RGBA16
+            | Self::RGBA16F
             | Self::RGB16
             | Self::RGBA8
+            | Self::SRGBA8
             | Self::RGB8
+            | Self::SRGB8
             | Self::BGRA8
             | Self::BGR8
             | Self::RG16
@@ -174,34 +186,42 @@ impl PixelKind {
             | Self::F16
             | Self::R8
             | Self::R8UI
-            | Self::RGBA32F => false,
+            | Self::RGBA32F
+            | Self::R11G11B10F
+            | Self::RGB10A2 => false,
         }
     }
 
     pub fn element_kind(self) -> PixelElementKind {
         match self {
-            PixelKind::F32 | PixelKind::F16 | PixelKind::RGBA32F | PixelKind::D32F => {
-                PixelElementKind::Float
-            }
-            PixelKind::D16
-            | PixelKind::D24S8
-            | PixelKind::RGBA8
-            | PixelKind::RGB8
-            | PixelKind::BGRA8
-            | PixelKind::BGR8
-            | PixelKind::RG8
-            | PixelKind::RG16
-            | PixelKind::R8
-            | PixelKind::R16
-            | PixelKind::RGB16
-            | PixelKind::RGBA16
-            | PixelKind::DXT1RGB
-            | PixelKind::DXT1RGBA
-            | PixelKind::DXT3RGBA
-            | PixelKind::DXT5RGBA
-            | PixelKind::R8RGTC
-            | PixelKind::RG8RGTC => PixelElementKind::NormalizedUnsignedInteger,
-            PixelKind::R8UI => PixelElementKind::UnsignedInteger,
+            Self::F32
+            | Self::F16
+            | Self::RGBA32F
+            | Self::RGBA16F
+            | Self::D32F
+            | Self::R11G11B10F => PixelElementKind::Float,
+            Self::D16
+            | Self::D24S8
+            | Self::RGBA8
+            | Self::SRGBA8
+            | Self::RGB8
+            | Self::SRGB8
+            | Self::BGRA8
+            | Self::BGR8
+            | Self::RG8
+            | Self::RG16
+            | Self::R8
+            | Self::R16
+            | Self::RGB16
+            | Self::RGBA16
+            | Self::DXT1RGB
+            | Self::DXT1RGBA
+            | Self::DXT3RGBA
+            | Self::DXT5RGBA
+            | Self::R8RGTC
+            | Self::RG8RGTC
+            | Self::RGB10A2 => PixelElementKind::NormalizedUnsignedInteger,
+            Self::R8UI => PixelElementKind::UnsignedInteger,
         }
     }
 }
@@ -229,15 +249,18 @@ fn image_3d_size_bytes(pixel_kind: PixelKind, width: usize, height: usize, depth
     let pixel_count = width * height * depth;
     match pixel_kind {
         PixelKind::RGBA32F => 16 * pixel_count,
-        PixelKind::RGBA16 => 8 * pixel_count,
+        PixelKind::RGBA16 | PixelKind::RGBA16F => 8 * pixel_count,
         PixelKind::RGB16 => 6 * pixel_count,
         PixelKind::RGBA8
+        | PixelKind::SRGBA8
         | PixelKind::BGRA8
         | PixelKind::RG16
         | PixelKind::D24S8
         | PixelKind::D32F
-        | PixelKind::F32 => 4 * pixel_count,
-        PixelKind::RGB8 | PixelKind::BGR8 => 3 * pixel_count,
+        | PixelKind::F32
+        | PixelKind::R11G11B10F
+        | PixelKind::RGB10A2 => 4 * pixel_count,
+        PixelKind::RGB8 | PixelKind::SRGB8 | PixelKind::BGR8 => 3 * pixel_count,
         PixelKind::RG8 | PixelKind::R16 | PixelKind::D16 | PixelKind::F16 => 2 * pixel_count,
         PixelKind::R8 | PixelKind::R8UI => pixel_count,
         PixelKind::DXT1RGB | PixelKind::DXT1RGBA | PixelKind::R8RGTC => {
@@ -255,15 +278,18 @@ fn image_2d_size_bytes(pixel_kind: PixelKind, width: usize, height: usize) -> us
     let pixel_count = width * height;
     match pixel_kind {
         PixelKind::RGBA32F => 16 * pixel_count,
-        PixelKind::RGBA16 => 8 * pixel_count,
+        PixelKind::RGBA16 | PixelKind::RGBA16F => 8 * pixel_count,
         PixelKind::RGB16 => 6 * pixel_count,
         PixelKind::RGBA8
+        | PixelKind::SRGBA8
         | PixelKind::BGRA8
         | PixelKind::RG16
         | PixelKind::D24S8
         | PixelKind::D32F
-        | PixelKind::F32 => 4 * pixel_count,
-        PixelKind::RGB8 | PixelKind::BGR8 => 3 * pixel_count,
+        | PixelKind::F32
+        | PixelKind::R11G11B10F
+        | PixelKind::RGB10A2 => 4 * pixel_count,
+        PixelKind::RGB8 | PixelKind::SRGB8 | PixelKind::BGR8 => 3 * pixel_count,
         PixelKind::RG8 | PixelKind::R16 | PixelKind::D16 | PixelKind::F16 => 2 * pixel_count,
         PixelKind::R8 | PixelKind::R8UI => pixel_count,
         PixelKind::DXT1RGB | PixelKind::DXT1RGBA | PixelKind::R8RGTC => {
@@ -280,15 +306,18 @@ fn image_2d_size_bytes(pixel_kind: PixelKind, width: usize, height: usize) -> us
 fn image_1d_size_bytes(pixel_kind: PixelKind, length: usize) -> usize {
     match pixel_kind {
         PixelKind::RGBA32F => 16 * length,
-        PixelKind::RGBA16 => 8 * length,
+        PixelKind::RGBA16 | PixelKind::RGBA16F => 8 * length,
         PixelKind::RGB16 => 6 * length,
         PixelKind::RGBA8
+        | PixelKind::SRGBA8
         | PixelKind::BGRA8
         | PixelKind::RG16
         | PixelKind::D24S8
         | PixelKind::D32F
-        | PixelKind::F32 => 4 * length,
-        PixelKind::RGB8 | PixelKind::BGR8 => 3 * length,
+        | PixelKind::F32
+        | PixelKind::R11G11B10F
+        | PixelKind::RGB10A2 => 4 * length,
+        PixelKind::RGB8 | PixelKind::SRGB8 | PixelKind::BGR8 => 3 * length,
         PixelKind::RG8 | PixelKind::R16 | PixelKind::D16 | PixelKind::F16 => 2 * length,
         PixelKind::R8 | PixelKind::R8UI => length,
         PixelKind::DXT1RGB | PixelKind::DXT1RGBA | PixelKind::R8RGTC => {
@@ -595,7 +624,8 @@ impl<'a> TextureBinding<'a> {
         let target = kind.gl_texture_target();
 
         unsafe {
-            self.state.set_texture(0, target, self.texture.texture);
+            self.state
+                .set_texture(0, target, Some(self.texture.texture));
 
             let (type_, format, internal_format) = match pixel_kind {
                 PixelKind::F32 => (glow::FLOAT, glow::RED, glow::R32F),
@@ -612,7 +642,9 @@ impl<'a> TextureBinding<'a> {
                     glow::DEPTH24_STENCIL8,
                 ),
                 PixelKind::RGBA8 => (glow::UNSIGNED_BYTE, glow::RGBA, glow::RGBA8),
+                PixelKind::SRGBA8 => (glow::UNSIGNED_BYTE, glow::RGBA, glow::SRGB8_ALPHA8),
                 PixelKind::RGB8 => (glow::UNSIGNED_BYTE, glow::RGB, glow::RGB8),
+                PixelKind::SRGB8 => (glow::UNSIGNED_BYTE, glow::RGB, glow::SRGB8),
                 PixelKind::RG8 => (glow::UNSIGNED_BYTE, glow::RG, glow::RG8),
                 PixelKind::R8 => (glow::UNSIGNED_BYTE, glow::RED, glow::R8),
                 PixelKind::R8UI => (glow::UNSIGNED_BYTE, glow::RED_INTEGER, glow::R8UI),
@@ -622,6 +654,11 @@ impl<'a> TextureBinding<'a> {
                 PixelKind::R16 => (glow::UNSIGNED_SHORT, glow::RED, glow::R16),
                 PixelKind::RGB16 => (glow::UNSIGNED_SHORT, glow::RGB, glow::RGB16),
                 PixelKind::RGBA16 => (glow::UNSIGNED_SHORT, glow::RGBA, glow::RGBA16),
+                PixelKind::RGB10A2 => (
+                    glow::UNSIGNED_INT_2_10_10_10_REV,
+                    glow::RGBA,
+                    glow::RGB10_A2,
+                ),
                 PixelKind::DXT1RGB => (0, 0, GL_COMPRESSED_RGB_S3TC_DXT1_EXT),
                 PixelKind::DXT1RGBA => (0, 0, GL_COMPRESSED_RGBA_S3TC_DXT1_EXT),
                 PixelKind::DXT3RGBA => (0, 0, GL_COMPRESSED_RGBA_S3TC_DXT3_EXT),
@@ -629,16 +666,16 @@ impl<'a> TextureBinding<'a> {
                 PixelKind::R8RGTC => (0, 0, COMPRESSED_RED_RGTC1),
                 PixelKind::RG8RGTC => (0, 0, COMPRESSED_RG_RGTC2),
                 PixelKind::RGBA32F => (glow::FLOAT, glow::RGBA, glow::RGBA32F),
+                PixelKind::RGBA16F => (glow::FLOAT, glow::RGBA, glow::RGBA16F),
+                PixelKind::R11G11B10F => (glow::FLOAT, glow::RGB, glow::R11F_G11F_B10F),
             };
 
             let is_compressed = pixel_kind.is_compressed();
 
-            // Compressed textures does not have such term as "unpack alignment", so we have to check
-            // for compressed textures here.
-            if !is_compressed {
+            if let Some(alignment) = pixel_kind.unpack_alignment() {
                 self.state
                     .gl
-                    .pixel_store_i32(glow::UNPACK_ALIGNMENT, pixel_kind.unpack_alignment());
+                    .pixel_store_i32(glow::UNPACK_ALIGNMENT, alignment);
             }
 
             let mut mip_byte_offset = 0;
@@ -901,7 +938,11 @@ impl GpuTexture {
         state: &'a mut PipelineState,
         sampler_index: u32,
     ) -> TextureBinding<'a> {
-        state.set_texture(sampler_index, self.kind.gl_texture_target(), self.texture);
+        state.set_texture(
+            sampler_index,
+            self.kind.gl_texture_target(),
+            Some(self.texture),
+        );
         TextureBinding {
             state,
             texture: self,
@@ -909,7 +950,11 @@ impl GpuTexture {
     }
 
     pub fn bind(&self, state: &mut PipelineState, sampler_index: u32) {
-        state.set_texture(sampler_index, self.kind.gl_texture_target(), self.texture);
+        state.set_texture(
+            sampler_index,
+            self.kind.gl_texture_target(),
+            Some(self.texture),
+        );
     }
 
     pub fn kind(&self) -> GpuTextureKind {
