@@ -30,6 +30,7 @@ use crate::{
 };
 use ddsfile::{Caps2, D3DFormat};
 use image::{ColorType, DynamicImage, GenericImageView, ImageError, ImageFormat};
+use std::fmt::{Debug, Formatter};
 use std::{
     borrow::Cow,
     collections::hash_map::DefaultHasher,
@@ -143,12 +144,47 @@ impl Visit for TextureKind {
     }
 }
 
+#[derive(Default, Clone)]
+struct TextureBytes(Vec<u8>);
+
+impl Visit for TextureBytes {
+    fn visit(&mut self, name: &str, visitor: &mut Visitor) -> VisitResult {
+        self.0.visit(name, visitor)
+    }
+}
+
+impl Debug for TextureBytes {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        write!(f, "Texture has {} bytes", self.0.len())
+    }
+}
+
+impl From<Vec<u8>> for TextureBytes {
+    fn from(bytes: Vec<u8>) -> Self {
+        Self(bytes)
+    }
+}
+
+impl Deref for TextureBytes {
+    type Target = Vec<u8>;
+
+    fn deref(&self) -> &Self::Target {
+        &self.0
+    }
+}
+
+impl DerefMut for TextureBytes {
+    fn deref_mut(&mut self) -> &mut Self::Target {
+        &mut self.0
+    }
+}
+
 /// Actual texture data.
 #[derive(Debug)]
 pub struct TextureData {
     path: PathBuf,
     kind: TextureKind,
-    bytes: Vec<u8>,
+    bytes: TextureBytes,
     pixel_kind: TexturePixelKind,
     minification_filter: TextureMinificationFilter,
     magnification_filter: TextureMagnificationFilter,
@@ -158,6 +194,7 @@ pub struct TextureData {
     anisotropy: f32,
     serialize_content: bool,
     data_hash: u64,
+    is_render_target: bool,
 }
 
 impl ResourceData for TextureData {
@@ -213,7 +250,7 @@ impl Default for TextureData {
                 width: 0,
                 height: 0,
             },
-            bytes: Vec::new(),
+            bytes: Default::default(),
             pixel_kind: TexturePixelKind::RGBA8,
             minification_filter: TextureMinificationFilter::LinearMipMapLinear,
             magnification_filter: TextureMagnificationFilter::Linear,
@@ -223,12 +260,13 @@ impl Default for TextureData {
             anisotropy: 16.0,
             serialize_content: false,
             data_hash: 0,
+            is_render_target: false,
         }
     }
 }
 
 define_new_resource!(
-    #[doc = "See module docs."],
+    /// See module docs.
     Texture<TextureData, TextureError>
 );
 
@@ -244,7 +282,7 @@ impl Texture {
             path: Default::default(),
             // Render target will automatically set width and height before rendering.
             kind: TextureKind::Rectangle { width, height },
-            bytes: Vec::new(),
+            bytes: Default::default(),
             pixel_kind: TexturePixelKind::RGBA8,
             minification_filter: TextureMinificationFilter::Linear,
             magnification_filter: TextureMagnificationFilter::Linear,
@@ -254,6 +292,7 @@ impl Texture {
             anisotropy: 1.0,
             serialize_content: false,
             data_hash: 0,
+            is_render_target: true,
         })))
     }
 
@@ -532,15 +571,19 @@ impl TexturePixelKind {
 }
 
 /// An error that may occur during texture operations.
-#[derive(Debug)]
+#[derive(Debug, thiserror::Error)]
 pub enum TextureError {
     /// Format (pixel format, dimensions) is not supported.
+    #[error("Unsupported format!")]
     UnsupportedFormat,
     /// An io error.
+    #[error("An i/o error has occurred: {0}")]
     Io(std::io::Error),
     /// Internal image crate error.
+    #[error("Image loading error {0}")]
     Image(image::ImageError),
     /// An error occurred during file loading.
+    #[error("A file load error has occurred {0:?}")]
     FileLoadError(FileLoadError),
 }
 
@@ -716,7 +759,7 @@ impl TextureData {
                 s_wrap_mode: TextureWrapMode::Repeat,
                 t_wrap_mode: TextureWrapMode::Repeat,
                 mip_count,
-                bytes,
+                bytes: bytes.into(),
                 kind: if dds.header.caps2 & Caps2::CUBEMAP == Caps2::CUBEMAP {
                     TextureKind::Cube {
                         width: dds.header.width,
@@ -766,7 +809,7 @@ impl TextureData {
                 pixel_kind: kind,
                 kind: TextureKind::Rectangle { width, height },
                 data_hash: data_hash(&bytes),
-                bytes,
+                bytes: bytes.into(),
                 ..Default::default()
             }
         };
@@ -778,29 +821,31 @@ impl TextureData {
 
             match (texture.pixel_kind, compression) {
                 (TexturePixelKind::RGB8, CompressionOptions::Speed) => {
-                    texture.bytes = compress_bc1::<tbc::color::Rgb8>(&texture.bytes, w, h);
+                    texture.bytes = compress_bc1::<tbc::color::Rgb8>(&texture.bytes, w, h).into();
                     texture.pixel_kind = TexturePixelKind::DXT1RGB;
                 }
                 (TexturePixelKind::RGB8, CompressionOptions::Quality) => {
-                    texture.bytes = compress_bc3::<tbc::color::Rgb8>(&texture.bytes, w, h);
+                    texture.bytes = compress_bc3::<tbc::color::Rgb8>(&texture.bytes, w, h).into();
                     texture.pixel_kind = TexturePixelKind::DXT5RGBA;
                 }
                 (TexturePixelKind::RGBA8, CompressionOptions::Speed) => {
-                    texture.bytes = compress_bc1::<tbc::color::Rgba8>(&texture.bytes, w, h);
+                    texture.bytes = compress_bc1::<tbc::color::Rgba8>(&texture.bytes, w, h).into();
                     texture.pixel_kind = TexturePixelKind::DXT1RGBA;
                 }
                 (TexturePixelKind::RGBA8, CompressionOptions::Quality) => {
-                    texture.bytes = compress_bc3::<tbc::color::Rgba8>(&texture.bytes, w, h);
+                    texture.bytes = compress_bc3::<tbc::color::Rgba8>(&texture.bytes, w, h).into();
                     texture.pixel_kind = TexturePixelKind::DXT5RGBA;
                 }
                 (TexturePixelKind::R8, CompressionOptions::Speed)
                 | (TexturePixelKind::R8, CompressionOptions::Quality) => {
-                    texture.bytes = compress_r8_bc4::<tbc::color::Red8>(&texture.bytes, w, h);
+                    texture.bytes =
+                        compress_r8_bc4::<tbc::color::Red8>(&texture.bytes, w, h).into();
                     texture.pixel_kind = TexturePixelKind::R8RGTC;
                 }
                 (TexturePixelKind::RG8, CompressionOptions::Speed)
                 | (TexturePixelKind::RG8, CompressionOptions::Quality) => {
-                    texture.bytes = compress_rg8_bc4::<tbc::color::RedGreen8>(&texture.bytes, w, h);
+                    texture.bytes =
+                        compress_rg8_bc4::<tbc::color::RedGreen8>(&texture.bytes, w, h).into();
                     texture.pixel_kind = TexturePixelKind::RG8RGTC;
                 }
                 _ => (),
@@ -893,7 +938,7 @@ impl TextureData {
                 path: Default::default(),
                 kind,
                 data_hash: data_hash(&bytes),
-                bytes,
+                bytes: bytes.into(),
                 pixel_kind,
                 serialize_content,
                 ..Default::default()
@@ -966,6 +1011,22 @@ impl TextureData {
         &self.bytes
     }
 
+    /// Returns true if the texture is procedural, false - otherwise.
+    ///
+    /// # Notes
+    ///
+    /// Content of procedural textures is saved during serialization and they never resolved
+    /// on deserialization. Resolving here means a process of getting correct texture instance
+    /// by its path.
+    pub fn is_procedural(&self) -> bool {
+        self.serialize_content
+    }
+
+    /// Returns true if the texture is used as render target.
+    pub fn is_render_target(&self) -> bool {
+        self.is_render_target
+    }
+
     /// Max samples for anisotropic filtering. Default value is 16.0 (max).
     /// However real value passed to GPU will be clamped to maximum supported
     /// by current GPU. To disable anisotropic filtering set this to 1.0.
