@@ -4,29 +4,27 @@
 //! Surfaces can use the same data source across many instances, this is a memory optimization for
 //! being able to re-use data when you need to draw the same mesh in many places.
 
-use crate::scene::mesh::buffer::{GeometryBuffer, VertexFetchError};
-use crate::scene::mesh::vertex::OldVertex;
 use crate::{
     core::{
         algebra::{Matrix4, Point3, Vector2, Vector3, Vector4},
-        color::Color,
         math::TriangleDefinition,
         pool::{ErasedHandle, Handle},
         visitor::{Visit, VisitResult, Visitor},
     },
-    resource::texture::Texture,
+    material::Material,
     scene::{
         mesh::{
             buffer::{
-                VertexAttributeDescriptor, VertexAttributeUsage, VertexBuffer, VertexReadTrait,
-                VertexWriteTrait,
+                GeometryBuffer, VertexAttributeDescriptor, VertexAttributeUsage, VertexBuffer,
+                VertexFetchError, VertexReadTrait, VertexWriteTrait,
             },
-            vertex::StaticVertex,
+            vertex::{OldVertex, StaticVertex},
         },
         node::Node,
     },
     utils::raw_mesh::{RawMesh, RawMeshBuilder},
 };
+use std::sync::Mutex;
 use std::{
     collections::hash_map::DefaultHasher,
     hash::{Hash, Hasher},
@@ -917,17 +915,12 @@ impl VertexWeightSet {
 }
 
 /// See module docs.
-#[derive(Debug, Default)]
+#[derive(Debug, Clone)]
 pub struct Surface {
     // Wrapped into option to be able to implement Default for serialization.
     // In normal conditions it must never be None!
     data: Option<Arc<RwLock<SurfaceData>>>,
-    diffuse_texture: Option<Texture>,
-    normal_texture: Option<Texture>,
-    lightmap_texture: Option<Texture>,
-    specular_texture: Option<Texture>,
-    roughness_texture: Option<Texture>,
-    height_texture: Option<Texture>,
+    material: Arc<Mutex<Material>>,
     /// Temporal array for FBX conversion needs, it holds skinning data (weight + bone handle)
     /// and will be used to fill actual bone indices and weight in vertices that will be
     /// sent to GPU. The idea is very simple: GPU needs to know only indices of matrices of
@@ -937,28 +930,15 @@ pub struct Surface {
     pub vertex_weights: Vec<VertexWeightSet>,
     /// Array of handle to scene nodes which are used as bones.
     pub bones: Vec<Handle<Node>>,
-    color: Color,
 }
 
-/// Shallow copy of surface.
-///
-/// # Notes
-///
-/// Handles to bones must be remapped afterwards, so it is not advised
-/// to use this clone to clone surfaces.
-impl Clone for Surface {
-    fn clone(&self) -> Self {
+impl Default for Surface {
+    fn default() -> Self {
         Self {
-            data: self.data.clone(),
-            diffuse_texture: self.diffuse_texture.clone(),
-            normal_texture: self.normal_texture.clone(),
-            specular_texture: self.specular_texture.clone(),
-            roughness_texture: self.roughness_texture.clone(),
-            height_texture: self.height_texture.clone(),
-            bones: self.bones.clone(),
-            vertex_weights: Vec::new(), // Intentionally not copied.
-            color: self.color,
-            lightmap_texture: self.lightmap_texture.clone(),
+            data: None,
+            material: Arc::new(Mutex::new(Material::standard())),
+            vertex_weights: Default::default(),
+            bones: Default::default(),
         }
     }
 }
@@ -969,40 +949,13 @@ impl Surface {
     pub fn new(data: Arc<RwLock<SurfaceData>>) -> Self {
         Self {
             data: Some(data),
-            diffuse_texture: None,
-            normal_texture: None,
-            specular_texture: None,
-            roughness_texture: None,
-            height_texture: None,
-            bones: Vec::new(),
-            vertex_weights: Vec::new(),
-            color: Color::WHITE,
-            lightmap_texture: None,
+            ..Default::default()
         }
     }
 
     /// Calculates batch id.
     pub fn batch_id(&self) -> u64 {
-        let mut hasher = DefaultHasher::new();
-
-        let data_key = &*self.data() as *const _ as u64;
-        data_key.hash(&mut hasher);
-
-        for texture in [
-            self.diffuse_texture.as_ref(),
-            self.normal_texture.as_ref(),
-            self.specular_texture.as_ref(),
-            self.roughness_texture.as_ref(),
-            self.lightmap_texture.as_ref(),
-            self.height_texture.as_ref(),
-        ]
-        .iter()
-        .filter_map(|t| *t)
-        {
-            texture.key().hash(&mut hasher);
-        }
-
-        hasher.finish()
+        &*self.material as *const _ as u64
     }
 
     /// Returns current data used by surface.
@@ -1011,126 +964,14 @@ impl Surface {
         self.data.as_ref().unwrap().clone()
     }
 
-    /// Sets new diffuse texture.
-    #[inline]
-    pub fn set_diffuse_texture(&mut self, tex: Option<Texture>) {
-        self.diffuse_texture = tex;
+    /// Returns current material of the surface.
+    pub fn material(&self) -> &Arc<Mutex<Material>> {
+        &self.material
     }
 
-    /// Returns current diffuse texture.
-    #[inline]
-    pub fn diffuse_texture(&self) -> Option<Texture> {
-        self.diffuse_texture.clone()
-    }
-
-    /// Returns current diffuse texture ref.
-    #[inline]
-    pub fn diffuse_texture_ref(&self) -> Option<&Texture> {
-        self.diffuse_texture.as_ref()
-    }
-
-    /// Sets new normal map texture.
-    #[inline]
-    pub fn set_normal_texture(&mut self, tex: Option<Texture>) {
-        self.normal_texture = tex;
-    }
-
-    /// Returns current normal map texture.
-    #[inline]
-    pub fn normal_texture(&self) -> Option<Texture> {
-        self.normal_texture.clone()
-    }
-
-    /// Returns current normal map texture by ref.
-    #[inline]
-    pub fn normal_texture_ref(&self) -> Option<&Texture> {
-        self.normal_texture.as_ref()
-    }
-
-    /// Sets new specular texture.
-    #[inline]
-    pub fn set_specular_texture(&mut self, tex: Option<Texture>) {
-        self.specular_texture = tex;
-    }
-
-    /// Returns current specular texture.
-    #[inline]
-    pub fn specular_texture(&self) -> Option<Texture> {
-        self.specular_texture.clone()
-    }
-
-    /// Returns current specular texture by ref.
-    #[inline]
-    pub fn specular_texture_ref(&self) -> Option<&Texture> {
-        self.specular_texture.as_ref()
-    }
-
-    /// Sets new roughness texture.
-    #[inline]
-    pub fn set_roughness_texture(&mut self, tex: Option<Texture>) {
-        self.roughness_texture = tex;
-    }
-
-    /// Returns current roughness texture.
-    #[inline]
-    pub fn roughness_texture(&self) -> Option<Texture> {
-        self.roughness_texture.clone()
-    }
-
-    /// Returns current roughness texture by ref.
-    #[inline]
-    pub fn roughness_texture_ref(&self) -> Option<&Texture> {
-        self.roughness_texture.as_ref()
-    }
-
-    /// Sets new lightmap texture.
-    #[inline]
-    pub fn set_lightmap_texture(&mut self, tex: Option<Texture>) {
-        self.lightmap_texture = tex;
-    }
-
-    /// Returns lightmap texture.
-    #[inline]
-    pub fn lightmap_texture(&self) -> Option<Texture> {
-        self.lightmap_texture.clone()
-    }
-
-    /// Returns lightmap texture.
-    #[inline]
-    pub fn lightmap_texture_ref(&self) -> Option<&Texture> {
-        self.lightmap_texture.as_ref()
-    }
-
-    /// Sets new height texture.
-    #[inline]
-    pub fn set_height_texture(&mut self, tex: Option<Texture>) {
-        self.height_texture = tex;
-    }
-
-    /// Returns height texture.
-    #[inline]
-    pub fn height_texture(&self) -> Option<Texture> {
-        self.height_texture.clone()
-    }
-
-    /// Returns height texture by ref.
-    #[inline]
-    pub fn height_texture_ref(&self) -> Option<&Texture> {
-        self.height_texture.as_ref()
-    }
-
-    /// Sets color of surface. Keep in mind that alpha component is **not** compatible
-    /// with deferred render path. You have to use forward render path if you need
-    /// transparent surfaces.
-    #[inline]
-    pub fn set_color(&mut self, color: Color) {
-        self.color = color;
-    }
-
-    /// Returns current color of surface.
-    #[inline]
-    pub fn color(&self) -> Color {
-        self.color
+    /// Sets new material for the surface.
+    pub fn set_material(&mut self, material: Arc<Mutex<Material>>) {
+        self.material = material;
     }
 
     /// Returns list of bones that affects the surface.
@@ -1144,16 +985,10 @@ impl Visit for Surface {
     fn visit(&mut self, name: &str, visitor: &mut Visitor) -> VisitResult {
         visitor.enter_region(name)?;
 
-        self.data.visit("Data", visitor)?;
-        self.normal_texture.visit("NormalTexture", visitor)?;
-        self.diffuse_texture.visit("DiffuseTexture", visitor)?;
-        self.specular_texture.visit("SpecularTexture", visitor)?;
-        self.roughness_texture.visit("RoughnessTexture", visitor)?;
-        self.height_texture.visit("HeightTexture", visitor)?;
-        self.color.visit("Color", visitor)?;
-        self.bones.visit("Bones", visitor)?;
-        self.lightmap_texture.visit("LightmapTexture", visitor)?;
         // self.vertex_weights intentionally not serialized!
+        self.data.visit("Data", visitor)?;
+        self.bones.visit("Bones", visitor)?;
+        let _ = self.material.visit("Material", visitor); // Backward compatibility.
 
         visitor.leave_region()
     }
@@ -1162,14 +997,8 @@ impl Visit for Surface {
 /// Surface builder allows you to create surfaces in declarative manner.
 pub struct SurfaceBuilder {
     data: Arc<RwLock<SurfaceData>>,
-    diffuse_texture: Option<Texture>,
-    normal_texture: Option<Texture>,
-    lightmap_texture: Option<Texture>,
-    specular_texture: Option<Texture>,
-    roughness_texture: Option<Texture>,
-    height_texture: Option<Texture>,
+    material: Option<Arc<Mutex<Material>>>,
     bones: Vec<Handle<Node>>,
-    color: Color,
 }
 
 impl SurfaceBuilder {
@@ -1177,56 +1006,14 @@ impl SurfaceBuilder {
     pub fn new(data: Arc<RwLock<SurfaceData>>) -> Self {
         Self {
             data,
-            diffuse_texture: None,
-            normal_texture: None,
-            lightmap_texture: None,
-            specular_texture: None,
-            roughness_texture: None,
-            height_texture: None,
+            material: None,
             bones: Default::default(),
-            color: Color::WHITE,
         }
     }
 
     /// Sets desired diffuse texture.
-    pub fn with_diffuse_texture(mut self, tex: Texture) -> Self {
-        self.diffuse_texture = Some(tex);
-        self
-    }
-
-    /// Sets desired normal map texture.
-    pub fn with_normal_texture(mut self, tex: Texture) -> Self {
-        self.normal_texture = Some(tex);
-        self
-    }
-
-    /// Sets desired lightmap texture.
-    pub fn with_lightmap_texture(mut self, tex: Texture) -> Self {
-        self.lightmap_texture = Some(tex);
-        self
-    }
-
-    /// Sets desired specular texture.
-    pub fn with_specular_texture(mut self, tex: Texture) -> Self {
-        self.specular_texture = Some(tex);
-        self
-    }
-
-    /// Sets desired roughness texture.
-    pub fn with_roughness_texture(mut self, tex: Texture) -> Self {
-        self.roughness_texture = Some(tex);
-        self
-    }
-
-    /// Sets desired roughness texture.
-    pub fn with_height_texture(mut self, tex: Texture) -> Self {
-        self.height_texture = Some(tex);
-        self
-    }
-
-    /// Sets desired color of surface.
-    pub fn with_color(mut self, color: Color) -> Self {
-        self.color = color;
+    pub fn with_material(mut self, material: Arc<Mutex<Material>>) -> Self {
+        self.material = Some(material);
         self
     }
 
@@ -1240,15 +1027,11 @@ impl SurfaceBuilder {
     pub fn build(self) -> Surface {
         Surface {
             data: Some(self.data),
-            diffuse_texture: self.diffuse_texture,
-            normal_texture: self.normal_texture,
-            lightmap_texture: self.lightmap_texture,
-            specular_texture: self.specular_texture,
-            roughness_texture: self.roughness_texture,
-            height_texture: self.height_texture,
+            material: self
+                .material
+                .unwrap_or_else(|| Arc::new(Mutex::new(Material::standard()))),
             vertex_weights: Default::default(),
             bones: self.bones,
-            color: self.color,
         }
     }
 }
