@@ -212,11 +212,13 @@ impl Deref for ProgressIndicator {
 }
 
 /// An error that may occur during ligthmap generation.
-#[derive(Debug)]
+#[derive(Debug, thiserror::Error)]
 pub enum LightmapGenerationError {
     /// Generation was cancelled by user.
+    #[error("Lightmap generation was cancelled by the user.")]
     Cancelled,
-    /// Vertex buffer of a mesh lacks needed data.
+    /// Vertex buffer of a mesh lacks required data.
+    #[error("Vertex buffer of a mesh lacks required data {0}.")]
     InvalidData(VertexFetchError),
 }
 
@@ -277,7 +279,7 @@ impl Lightmap {
                             .up_vector()
                             .try_normalize(std::f32::EPSILON)
                             .unwrap_or_else(Vector3::y),
-                        color: light.color().as_frgb(),
+                        color: light.color().srgb_to_linear().as_frgb(),
                     }))
                 }
                 Light::Spot(spot) => lights.push(LightDefinition::Spot(SpotLightDefinition {
@@ -285,7 +287,7 @@ impl Lightmap {
                     intensity: 1.0,
                     edge0: ((spot.hotspot_cone_angle() + spot.falloff_angle_delta()) * 0.5).cos(),
                     edge1: (spot.hotspot_cone_angle() * 0.5).cos(),
-                    color: light.color().as_frgb(),
+                    color: light.color().srgb_to_linear().as_frgb(),
                     direction: light
                         .up_vector()
                         .try_normalize(std::f32::EPSILON)
@@ -298,7 +300,7 @@ impl Lightmap {
                     handle,
                     intensity: 1.0,
                     position: light.global_position(),
-                    color: light.color().as_frgb(),
+                    color: light.color().srgb_to_linear().as_frgb(),
                     radius: point.radius(),
                     sqr_radius: point.radius() * point.radius(),
                 })),
