@@ -447,57 +447,3 @@ pub fn generate_uvs_mesh(
 
     Ok(patches)
 }
-
-#[cfg(test)]
-mod test {
-    use crate::core::algebra::{Matrix4, Vector3};
-    use crate::scene::mesh::buffer::{VertexAttributeUsage, VertexReadTrait};
-    use crate::{scene::mesh::surface::SurfaceData, utils::uvgen::generate_uvs};
-    use image::{Rgb, RgbImage};
-    use imageproc::drawing::draw_line_segment_mut;
-
-    #[test]
-    fn test_generate_uvs() {
-        //let mut data = SurfaceSharedData::make_sphere(100, 100, 1.0);
-        //let mut data = SurfaceSharedData::make_cylinder(80, 1.0, 1.0, true, Matrix4::identity());
-        //let mut data = SurfaceSharedData::make_cube(Matrix4::identity());
-        let mut data = SurfaceData::make_cone(
-            16,
-            1.0,
-            1.0,
-            &Matrix4::new_nonuniform_scaling(&Vector3::new(1.0, 1.1, 1.0)),
-        );
-        generate_uvs(&mut data, 0.01).unwrap();
-
-        let white = Rgb([255u8, 255u8, 255u8]);
-        let mut image = RgbImage::new(1024, 1024);
-        for triangle in data.geometry_buffer.iter() {
-            let a = data
-                .vertex_buffer
-                .get(triangle[0] as usize)
-                .unwrap()
-                .read_2_f32(VertexAttributeUsage::TexCoord1)
-                .unwrap()
-                .scale(1024.0);
-            let b = data
-                .vertex_buffer
-                .get(triangle[1] as usize)
-                .unwrap()
-                .read_2_f32(VertexAttributeUsage::TexCoord1)
-                .unwrap()
-                .scale(1024.0);
-            let c = data
-                .vertex_buffer
-                .get(triangle[2] as usize)
-                .unwrap()
-                .read_2_f32(VertexAttributeUsage::TexCoord1)
-                .unwrap()
-                .scale(1024.0);
-
-            draw_line_segment_mut(&mut image, (a.x, a.y), (b.x, b.y), white);
-            draw_line_segment_mut(&mut image, (b.x, b.y), (c.x, c.y), white);
-            draw_line_segment_mut(&mut image, (c.x, c.y), (a.x, a.y), white);
-        }
-        image.save("uvgen.png").unwrap();
-    }
-}
